// Copyright (C) 2006 Pushok Software http://www.pushok.com/soft_svncom.php
//
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html
//
// Subversion is Copyrght of(c) 2000-2004 CollabNet.  All rights reserved.
// Full license available here http://subversion.tigris.org/license-1.html.

#include "stdafx.h"
#include "SVNAdmin.h"

#include ".\svnadmin.h"
#include "svn_fs.h"
#include "svn_repos.h"
#include "svn_opt.h"
#include "svn_config.h"

#include "Convert.h"

// CSVNAdmin

STDMETHODIMP CSVNAdmin::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_ISVNAdmin
	};

	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

#define BEGIN_SVN_CALL \
	HRESULT hr;\
	{\
	hr = S_OK;\
	apr_status_t status;\
	status = apr_initialize();\
	if (status) return AtlReportError(__uuidof(SVNAdmin),_T("Cannot initialize apr pool"));\
	apr_pool_t *pFuncPool = svn_pool_create(NULL);\
	try\
	{

#define END_SVN_CALL \
	}\
	catch (HRESULT _hr)\
	{\
	hr = _hr;\
	}\
	catch (...)\
	{\
	ATLASSERT(FALSE);\
	hr = E_UNEXPECTED;\
	}\
	svn_pool_destroy(pFuncPool);\
	}

#define SVN_CALL_ERROR(hr,msg) \
	do {\
	throw AtlReportError(__uuidof(SVNAdmin),CString(msg),__uuidof(ISVNAdmin),hr);\
	} while(false)

#define SVN_CALL_SVNERROR(err) \
	do {\
		svn_error_t *_err = err;\
		if (_err)\
			throw AtlReportError(__uuidof(SVNAdmin),Utf82Ole(_err->message),__uuidof(ISVNAdmin),hr);\
	} while(false)



STDMETHODIMP CSVNAdmin::CreateRepository(BSTR sReposPath, BSTR nReposType)
{
	BEGIN_SVN_CALL

	svn_repos_t *repos;
	apr_hash_t *config;
	apr_hash_t *fs_config = apr_hash_make (pFuncPool);;

	apr_hash_set (fs_config, SVN_FS_CONFIG_BDB_TXN_NOSYNC,
				APR_HASH_KEY_STRING, "0");

	apr_hash_set (fs_config, SVN_FS_CONFIG_BDB_LOG_AUTOREMOVE,
				APR_HASH_KEY_STRING, "1");

	CString sTemp = Ole2Utf8(nReposType);
	apr_hash_set (fs_config, SVN_FS_CONFIG_FS_TYPE,
					APR_HASH_KEY_STRING, sTemp);

	SVN_CALL_SVNERROR(svn_config_get_config (&config, NULL, pFuncPool));

	SVN_CALL_SVNERROR(svn_repos_create (&repos, Ole2Utf8(sReposPath),
								NULL, NULL, config, fs_config, pFuncPool));

	END_SVN_CALL
	return S_OK;
}


