// Copyright (C) 2006 Pushok Software http://www.pushok.com/soft_svncom.php
//
// LICENSE
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License (GPL)
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// To read the license please visit http://www.gnu.org/copyleft/gpl.html
//
// Subversion is Copyrght of(c) 2000-2004 CollabNet.  All rights reserved.
// Full license available here http://subversion.tigris.org/license-1.html.

#pragma once
#include "resource.h"       // main symbols

#include "pcom.h"


// CSVNWcPropList

class ATL_NO_VTABLE CSVNWcPropList : 
	public CComObjectRootEx<CComMultiThreadModel>,
	public CComCoClass<CSVNWcPropList, &CLSID_SVNWcPropList>,
	public ISupportErrorInfo,
	public IDispatchImpl<ISVNWcPropList, &IID_ISVNWcPropList, &LIBID_PushOkSvn, /*wMajor =*/ 1, /*wMinor =*/ 0>
{
public:
	CSVNWcPropList()
	{
		m_bFirst = true;
	}

DECLARE_REGISTRY_RESOURCEID(IDR_SVNWCPROPLIST)

BEGIN_COM_MAP(CSVNWcPropList)
	COM_INTERFACE_ENTRY(ISVNWcPropList)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY_AGGREGATE(IID_IMarshal, m_pUnkMarshaler.p)
END_COM_MAP()

// ISupportsErrorInfo
	STDMETHOD(InterfaceSupportsErrorInfo)(REFIID riid);

	DECLARE_PROTECT_FINAL_CONSTRUCT()
	DECLARE_GET_CONTROLLING_UNKNOWN()

	HRESULT FinalConstruct()
	{
		return CoCreateFreeThreadedMarshaler(
			GetControllingUnknown(), &m_pUnkMarshaler.p);
	}
	
	void FinalRelease() 
	{
		m_pUnkMarshaler.Release();
	}

	CComPtr<IUnknown> m_pUnkMarshaler;

public:
	STDMETHOD(put_Path)(BSTR sVal) {
		LOCK_THIS_OBJ
		m_Path = sVal;
		return S_OK;
	}

	STDMETHOD(get_Path)(BSTR* psVal) {
		LOCK_THIS_OBJ
		return m_Path.CopyTo(psVal);
	}

	STDMETHOD(get_CountProp)(LONG* pVal) {
		if (!pVal)
			return E_INVALIDARG;
		LOCK_THIS_OBJ
		*pVal = (LONG)m_PropMap.size();
		return S_OK;
	}
	STDMETHOD(AddProp)(BSTR sPropName, BSTR sPropVal) {
		LOCK_THIS_OBJ
		m_PropMap[sPropName] = sPropVal;
		m_bFirst = true;
		return S_OK;
	}
	STDMETHOD(GetProp) (BSTR sPropName, BSTR* sPropVal) {
		LOCK_THIS_OBJ
		if (m_PropMap.count(sPropName)) (m_PropMap[sPropName]).CopyTo(sPropVal);
		else CComBSTR("").CopyTo(sPropVal);
		return S_OK;
	}
	STDMETHOD(GetPropFirst) (BSTR* sPropName, BSTR* sPropVal) {
		LOCK_THIS_OBJ
		m_bFirst = false;
		m_Iter = m_PropMap.begin();
		if (!m_PropMap.size())
		{
			sPropName = NULL;
			sPropVal = NULL;
			return S_OK;
		}
		((CComBSTR)m_Iter->first).CopyTo(sPropName);
		(m_Iter->second).CopyTo(sPropVal);
		return S_OK;
	}
	STDMETHOD(GetPropNext) (BSTR* sPropName, BSTR* sPropVal) {
		LOCK_THIS_OBJ
		if (m_bFirst)
			return GetPropFirst(sPropName, sPropVal);
		if (m_Iter==m_PropMap.end())
		{
			sPropName = NULL;
			sPropVal = NULL;
			return S_OK;
		}
		m_Iter++;
		((CComBSTR)m_Iter->first).CopyTo(sPropName);
		(m_Iter->second).CopyTo(sPropVal);
		return S_OK;
	}
	STDMETHOD(Clear) (void) {
		LOCK_THIS_OBJ
		m_PropMap.clear();
		m_bFirst = true;
		return S_OK;
	}
   // Wrapper for GetPropFirst which returns data as SafeArray of Variants to be compatible with VBScript / JScript
	STDMETHOD(GetPropFirst2) (SAFEARRAY** pData) {
      CComVariant vPropName(""), vPropVal("");
      HRESULT hr;
      *pData = NULL;
      hr = GetPropFirst(&vPropName.bstrVal, &vPropVal.bstrVal);
      if (SUCCEEDED(hr))
      {
         CComSafeArray<VARIANT> arr(2);
         arr.SetAt(0, vPropName);
         arr.SetAt(1, vPropVal);
         *pData = arr.Detach();
      }
      return hr;
	}
   // Wrapper for GetPropNext which returns data as SafeArray of Variants to be compatible with VBScript / JScript
	STDMETHOD(GetPropNext2) (SAFEARRAY** pData) {
      CComVariant vPropName(""), vPropVal("");
      HRESULT hr;
      *pData = NULL;
      hr = GetPropNext(&vPropName.bstrVal, &vPropVal.bstrVal);
      if (SUCCEEDED(hr))
      {
         CComSafeArray<VARIANT> arr(2);
         arr.SetAt(0, vPropName);
         arr.SetAt(1, vPropVal);
         *pData = arr.Detach();
      }
      return hr;
	}


private:
	CComBSTR m_Path;
	std::map < CComBSTR, CComBSTR > m_PropMap;
	std::map < CComBSTR, CComBSTR >::iterator m_Iter;
	bool m_bFirst; // was added just to preserve behavior of GetPropNext();
	               // probably it can be completely removed.
};

OBJECT_ENTRY_AUTO(__uuidof(SVNWcPropList), CSVNWcPropList)

namespace PushokAtlEx
{
	template <>
	class GenericCopy< VARIANT, ISVNWcPropList* >
	{
	public :
		static void init(VARIANT* p)
		{
			GenericCopy<VARIANT>::init(p);
		}
		static void destroy(VARIANT* p)
		{
			GenericCopy<VARIANT>::destroy(p);
		}
		static HRESULT copy( VARIANT *pTo, ISVNWcPropList* *pFrom)
		{
			CComVariant vt((IDispatch *)*pFrom);
			return vt.Detach(pTo);
		}
	}; // class GenericCopy<VARIANT, ISVNWcNotify* pVal>
}

namespace SVNWcPropListVector_
{
	// Store our data in a vector of std::strings
	typedef PushokAtlEx::CComPtrAdapt<ISVNWcPropList> ItemType;
	typedef std::vector<ItemType> ContainerType;

	// Now we have all the PropListrmation we need to fill in the template arguments on the implementation classes
	typedef CComEnumOnSTL< IEnumVARIANT, &__uuidof(IEnumVARIANT), VARIANT,
							PushokAtlEx::GenericCopy< VARIANT, ItemType >, ContainerType > EnumeratorType;

	typedef ICollectionOnSTLImpl< ISVNWcPropListVector, ContainerType, ISVNWcPropList*,
							PushokAtlEx::GenericCopy<ISVNWcPropList*, ItemType>, EnumeratorType > CollectionType;
};

class ATL_NO_VTABLE CSVNWcPropListVector : 
	public CComObjectRootEx<CComMultiThreadModel>,
	public CComCoClass<CSVNWcPropListVector, &CLSID_SVNWcPropListVector>,
	public IDispatchImpl<SVNWcPropListVector_::CollectionType,&__uuidof(ISVNWcPropListVector)>
{
public:
	CSVNWcPropListVector()
	{
	}

DECLARE_REGISTRY_RESOURCEID(IDR_SVNWCPROPLISTVECTOR)

BEGIN_COM_MAP(CSVNWcPropListVector)
	COM_INTERFACE_ENTRY(ISVNWcPropListVector)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY_AGGREGATE(IID_IMarshal, m_pUnkMarshaler.p)
END_COM_MAP()


	DECLARE_PROTECT_FINAL_CONSTRUCT()
	DECLARE_GET_CONTROLLING_UNKNOWN()

	HRESULT FinalConstruct()
	{
		return CoCreateFreeThreadedMarshaler(
			GetControllingUnknown(), &m_pUnkMarshaler.p);
	}
	
	void FinalRelease() 
	{
		m_pUnkMarshaler.Release();
	}

	CComPtr<IUnknown> m_pUnkMarshaler;

public:
	STDMETHOD(Add)(ISVNWcPropList* pVal) {
		LOCK_THIS_OBJ
		m_coll.push_back(pVal);
		return S_OK;
	}
	STDMETHOD(Remove) (long Index) {
		LOCK_THIS_OBJ
		try {
			m_coll.erase(m_coll.begin() + Index);
		} catch(...) {
			return E_FAIL;
		}
		return S_OK;
	}
	STDMETHOD(Clear) (void) {
		LOCK_THIS_OBJ
		m_coll.clear();
		return S_OK;
	}
};

OBJECT_ENTRY_AUTO(__uuidof(SVNWcPropListVector), CSVNWcPropListVector)